﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ReflectionWinFormsApp
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void otworzToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // tworzy zmienną dialog, która ma metodę Dispose(), która będzie wywołana po wykonaniu kodu w klamerkach
            // w ten sposób zwalnia się zasób do okna dialogowego
            using (OpenFileDialog dialog = new OpenFileDialog())
            {
                dialog.Filter = "Pliki DLL|*.dll|Pliki EXE|*.exe|Wszystkie pliki (*.*)|*.*";
                if (dialog.ShowDialog(this) == DialogResult.OK)
                {
                    ZaladujDll(dialog.FileName);
                }
            }
        }

        private void ZaladujDll(string sciezkaPliku)
        {
            if (!File.Exists(sciezkaPliku))
            {
                MessageBox.Show("Podany plik nie istnieje!");
                return;
            }

            // załadowanie biblioteki/programu
            Assembly dll = null;
            try
            {
                dll = Assembly.LoadFrom(sciezkaPliku);
            }
            catch(Exception ex)
            {
                MessageBox.Show($"Wystąpił błąd podczas otwierania bibllioteki: {ex.Message}.");
                return;
            }

            // utworzenie głównego korzenia w drzewku
            TreeNode wezelDll = new TreeNode(dll.FullName);

            foreach (Type typ in dll.GetTypes())
            {
                // tworzenie i dodanie węzła z nazwą typu (klasa, interfejs, delegat)
                TreeNode wezelTyp = new TreeNode(typ.FullName);

                TreeNode wezelPola = new TreeNode("Pola");

                foreach (FieldInfo pole in typ.GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static))
                {
                    wezelPola.Nodes.Add((pole.IsPublic ? "publiczne" : "prywatne") + $" {pole.Name} ({pole.FieldType.FullName})");
                }

                // wezelPola.Nodes.AddRange(typ.GetFields().Select(pole => new TreeNode(pole.Name)).ToArray());

                TreeNode wezelWlasciowosci = new TreeNode("Właściwości");

                foreach (PropertyInfo wlasciwosc in typ.GetProperties(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static))
                {
                    wezelWlasciowosci.Nodes.Add($"{wlasciwosc.Name} ({wlasciwosc.PropertyType.FullName})");
                }

                TreeNode wezelMetody = new TreeNode("Metody");

                foreach (MethodInfo metoda in typ.GetMethods(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static))
                {
                    wezelMetody.Nodes.Add((metoda.IsPublic ? "publiczne" : "prywatne") + $" {metoda.Name} (zwraca: {metoda.ReturnType.FullName})");
                }

                wezelTyp.Nodes.AddRange(new[] { wezelPola, wezelWlasciowosci, wezelMetody });

                wezelDll.Nodes.Add(wezelTyp);
            }

            // dodanie głównego korzenia do kontrolki drzewka
            drzewko.Nodes.Add(wezelDll);
        }

        private void zakonczToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Czy na pewno chcesz zamknąć mój przekosmiczny program?", "Pytanie", MessageBoxButtons.OKCancel) == DialogResult.OK)
                Close();
        }

        private void wyczyscToolStripMenuItem_Click(object sender, EventArgs e)
        {
            drzewko.Nodes.Clear();
        }
    }
}
